﻿using System;

namespace NMock2.Monitoring
{
	/// <summary>
	/// Implementations of this interface are responsible for generating runtime
	/// proxies of classes and interfaces for use as mock objects.
	/// </summary>
	/// <remarks>
	/// Returned instances are expected to implement IMockObject and take care of
	/// intercepting calls to their public members. Intercepted calls should be
	/// forwarded on to the supplied Mockery for processing against expectations.
	/// </remarks>
	public interface IMockObjectFactory
	{
		/// <summary>
		/// Creates a mock of the specified class type.
		/// </summary>
		/// <param name="mockery">The mockery used to create this mock instance.</param>
		/// <param name="mockedType">The type of the mock to create.</param>
		/// <param name="name">The name to use for the mock instance.</param>
		/// <param name="mockStyle">The behaviour of the mock instance when first created.</param>
		/// <param name="constructorArgs">Constructor arguments for the class to be mocked.</param>
		/// <returns>A mock instance of the specified class type.</returns>
		object CreateMockForClass(Mockery mockery, Type mockedType, string name, MockStyle mockStyle, object[] constructorArgs);

		/// <summary>
		/// Creates a mock of the specified interface type.
		/// </summary>
		/// <param name="mockery">The mockery used to create this mock instance.</param>
		/// <param name="mockedType">The type of the mock to create.</param>
		/// <param name="name">The name to use for the mock instance.</param>
		/// <returns>A mock instance of the specified interface type.</returns>
		object CreateMockForInterface(Mockery mockery, Type mockedType, string name);
	}
}
